/*
 * File:   ble2.h
 * Author: choma and michael williams
 *
 * Created on March 31, 2015, 8:13 PM
 */

#ifndef BLE_H
#define	BLE_H

#include "services.h"
#include <xc.h>
#include "../configbitsrev2014vC-int.h"
#include "./NRFspi.h"
#include "../SDlib.h"
#include "lib_aci.h"
#include "aci_setup.h"
#include "uart_over_ble.h"

static hal_aci_data_t setup_msgs[NB_SETUP_MESSAGES]  = SETUP_MESSAGES_CONTENT;
static struct aci_state_t aci_state;

#ifdef SERVICES_PIPE_TYPE_MAPPING_CONTENT
    static services_pipe_type_mapping_t
        services_pipe_type_mapping[NUMBER_OF_PIPES] = SERVICES_PIPE_TYPE_MAPPING_CONTENT;
#else
    #define NUMBER_OF_PIPES 0
    static services_pipe_type_mapping_t * services_pipe_type_mapping = NULL;
#endif

static hal_aci_evt_t  aci_data;
static bool timing_change_done          = false;

static uart_over_ble_t uart_over_ble;
static uint8_t         uart_buffer[20];
static uint8_t         uart_buffer_len = 0;
static uint8_t         dummychar = 0;

void __ble_assert(const char *file, uint16_t line)
{
//  Serial.print("ERROR ");
//  Serial.print(file);
//  Serial.print(": ");
//  Serial.print(line);
//  Serial.print("\n");
  while(1);
}

void ble_setup(void);
bool loop(bool);


void ble_setup(void)
{
  serial_init(57600);
  printf("there was a print here\r\n");

  /* setup toggles on f0 and f1*/
  TRISFbits.TRISF0 = 0;
  TRISFbits.TRISF1 = 0;
#define blue LATFbits.LATF0 = 1; Nop(); LATFbits.LATF0 = 0;
#define vio LATFbits.LATF1 = 1; Nop(); LATFbits.LATF1 = 0;


  /**
  Point ACI data structures to the the setup data that the nRFgo studio generated for the nRF8001
  */
  if (NULL != services_pipe_type_mapping)
  {
    aci_state.aci_setup_info.services_pipe_type_mapping = &services_pipe_type_mapping[0];
  }
  else
  {
    aci_state.aci_setup_info.services_pipe_type_mapping = NULL;
  }
  aci_state.aci_setup_info.number_of_pipes    = NUMBER_OF_PIPES;
  aci_state.aci_setup_info.setup_msgs         = setup_msgs;  //????????
  aci_state.aci_setup_info.num_setup_msgs     = NB_SETUP_MESSAGES;

  /*
  Tell the ACI library, the MCU to nRF8001 pin connections.
  The Active pin is optional and can be marked UNUSED
  */

  aci_state.aci_pins.board_name = BOARD_DEFAULT; //See board.h for details REDBEARLAB_SHIELD_V1_1 or BOARD_DEFAULT
  aci_state.aci_pins.reqn_pin   = LATBbits.LATB8; //SPI chip select
  aci_state.aci_pins.rdyn_pin   = PORTDbits.RD0; //ready sig (also int!!)

  /* SPI hadled in hardware*/
//  aci_state.aci_pins.mosi_pin   = MOSI;
//  aci_state.aci_pins.miso_pin   = MISO;
//  aci_state.aci_pins.sck_pin    = SCK;
//
//  aci_state.aci_pins.spi_clock_divider      = SPI_CLOCK_DIV8;//SPI_CLOCK_DIV8  = 2MHz SPI speed
//                                                             //SPI_CLOCK_DIV16 = 1MHz SPI speed
//
  NRFspi_init(2000000ul);

  aci_state.aci_pins.reset_pin              = LATBbits.LATB12; //4 for Nordic board, UNUSED for REDBEARLAB_SHIELD_V1_1
  aci_state.aci_pins.active_pin             = LATBbits.LATB11;  // siad unused
  aci_state.aci_pins.optional_chip_sel_pin  = UNUSED;

  aci_state.aci_pins.interface_is_interrupt = false; //Interrupts still not available in Chipkit
  aci_state.aci_pins.interrupt_number       = 1;

  //We reset the nRF8001 here by toggling the RESET line connected to the nRF8001
  //If the RESET line is not available we call the ACI Radio Reset to soft reset the nRF8001
  //then we initialize the data structures required to setup the nRF8001
  //The second parameter is for turning debug printing on for the ACI Commands and Events so they be printed on the Serial
  lib_aci_init(&aci_state, false);
  printf("Setup done\r\n");
  blue
}

void uart_over_ble_init(void)
{
  uart_over_ble.uart_rts_local = true;
}

bool uart_tx(uint8_t *buffer, uint8_t buffer_len)
{
  bool status = false;

  if (lib_aci_is_pipe_available(&aci_state, PIPE_UART_OVER_BTLE_UART_TX_TX) &&
      (aci_state.data_credit_available >= 1))
  {
    status = lib_aci_send_data(PIPE_UART_OVER_BTLE_UART_TX_TX, buffer, buffer_len);
    if (status)
    {
      aci_state.data_credit_available--;
    }
  }

  return status;
}

bool uart_process_control_point_rx(uint8_t *byte, uint8_t length)
{
  bool status = false;
  aci_ll_conn_params_t *conn_params;

  if (lib_aci_is_pipe_available(&aci_state, PIPE_UART_OVER_BTLE_UART_CONTROL_POINT_TX) )
  {
  //  Serial.println(*byte, HEX);
    printf("%x\r\n", *byte);
    switch(*byte)
    {
      /*
      Queues a ACI Disconnect to the nRF8001 when this packet is received.
      May cause some of the UART packets being sent to be dropped
      */
      case UART_OVER_BLE_DISCONNECT:
        /*
        Parameters:
        None
        */
        lib_aci_disconnect(&aci_state, ACI_REASON_TERMINATE);
        status = true;
        break;


      /*
      Queues an ACI Change Timing to the nRF8001
      */
      case UART_OVER_BLE_LINK_TIMING_REQ:
        /*
        Parameters:
        Connection interval min: 2 bytes
        Connection interval max: 2 bytes
        Slave latency:           2 bytes
        Timeout:                 2 bytes
        Same format as Peripheral Preferred Connection Parameters (See nRFgo studio -> nRF8001 Configuration -> GAP Settings
        Refer to the ACI Change Timing Request in the nRF8001 Product Specifications
        */
        conn_params = (aci_ll_conn_params_t *)(byte+1);
        lib_aci_change_timing( conn_params->min_conn_interval,
                                conn_params->max_conn_interval,
                                conn_params->slave_latency,
                                conn_params->timeout_mult);
        status = true;
        break;

      /*
      Clears the RTS of the UART over BLE
      */
      case UART_OVER_BLE_TRANSMIT_STOP:
        /*
        Parameters:
        None
        */
        uart_over_ble.uart_rts_local = false;
        status = true;
        break;


      /*
      Set the RTS of the UART over BLE
      */
      case UART_OVER_BLE_TRANSMIT_OK:
        /*
        Parameters:
        None
        */
        uart_over_ble.uart_rts_local = true;
        status = true;
        break;
    }
  }

  return status;
}

int count = 0;

//bool aci_loop(bool completed)
//{
//  static bool setup_required = false;
//  bool sent = completed;
////  bool completed = false;
//
//  // We enter the if statement only when there is a ACI event available to be processed
//  if (lib_aci_event_get(&aci_state, &aci_data))
//  {
//    aci_evt_t * aci_evt;
//    aci_evt = &aci_data.evt;
//    switch(aci_evt->evt_opcode)
//    {
//      /**
//      As soon as you reset the nRF8001 you will get an ACI Device Started Event
//      */
//      case ACI_EVT_DEVICE_STARTED:
//      {
//        aci_state.data_credit_total = aci_evt->params.device_started.credit_available;
//        switch(aci_evt->params.device_started.device_mode)
//        {
//          case ACI_DEVICE_SETUP:
//            /**
//            When the device is in the setup mode
//            */
//            printf("Evt Device Started: Setup\r\n");
//            setup_required = true;
//            break;
//
//          case ACI_DEVICE_STANDBY:
//            printf("Evt Device Started: Standby\r\n");
//            //Looking for an iPhone by sending radio advertisements
//            //When an iPhone connects to us we will get an ACI_EVT_CONNECTED event from the nRF8001
//            if (aci_evt->params.device_started.hw_error)
//            {
//              delay_ms(20); //Handle the HW error event correctly.
//            }
//            else
//            {
//              lib_aci_connect(0/* in seconds : 0 means forever */, 0x0050 /* advertising interval 50ms*/);
//              printf("Advertising started : Tap Connect on the nRF UART app\r\n");
//            }
//
//            break;
//        }
//      }
//      break; //ACI Device Started Event
//
//      case ACI_EVT_CMD_RSP:
//        //If an ACI command response event comes with an error -> stop
//        if (ACI_STATUS_SUCCESS != aci_evt->params.cmd_rsp.cmd_status)
//        {
//          //ACI ReadDynamicData and ACI WriteDynamicData will have status codes of
//          //TRANSACTION_CONTINUE and TRANSACTION_COMPLETE
//          //all other ACI commands will have status code of ACI_STATUS_SCUCCESS for a successful command
////          Serial.print(F("ACI Command "));
////          Serial.println(aci_evt->params.cmd_rsp.cmd_opcode, HEX);
////          Serial.print(F("Evt Cmd respone: Status "));
////          Serial.println(aci_evt->params.cmd_rsp.cmd_status, HEX);
//
//          printf("ACI Command ");
//          printf("%x\r\n",aci_evt->params.cmd_rsp.cmd_opcode);
//          printf("Evt Cmd respone: Status ");
//          printf("%x\r\n",aci_evt->params.cmd_rsp.cmd_status);
//        }
//        if (ACI_CMD_GET_DEVICE_VERSION == aci_evt->params.cmd_rsp.cmd_opcode)
//        {
//          //Store the version and configuration information of the nRF8001 in the Hardware Revision String Characteristic
//          lib_aci_set_local_data(&aci_state, PIPE_DEVICE_INFORMATION_HARDWARE_REVISION_STRING_SET,
//            (uint8_t *)&(aci_evt->params.cmd_rsp.params.get_device_version), sizeof(aci_evt_cmd_rsp_params_get_device_version_t));
//        }
//        break;
//
//      case ACI_EVT_CONNECTED:
//          sent = false;
//        printf("Evt Connected");
//        uart_over_ble_init();
//        timing_change_done              = false;
//        aci_state.data_credit_available = aci_state.data_credit_total;
////        completed = false;
//        /*
//        Get the device version of the nRF8001 and store it in the Hardware Revision String
//        */
//        lib_aci_device_version();
//        break;
//
//      case ACI_EVT_PIPE_STATUS:
//        printf("Evt Pipe Status");
////        completed = false;
////        if (lib_aci_is_pipe_available(&aci_state, PIPE_UART_OVER_BTLE_UART_TX_TX) && (false == timing_change_done))
////        {
////          lib_aci_change_timing_GAP_PPCP(); // change the timing on the link as specified in the nRFgo studio -> nRF8001 conf. -> GAP.
////                                            // Used to increase or decrease bandwidth
////          timing_change_done = true;
////
//////          char hello[]="Hello World, works";
//////          uart_tx((uint8_t *)&hello[0], strlen(hello));
//////          printf("Sending :");
//////          printf("hello\r\n");
////        }
////         char data[] = "connected";
////        uart_tx((uint8_t *)&data, strlen(data));
//        break;
//
//      case ACI_EVT_TIMING:
//        printf("Evt link connection interval changed");
//        lib_aci_set_local_data(&aci_state,
//                                PIPE_UART_OVER_BTLE_UART_LINK_TIMING_CURRENT_SET,
//                                (uint8_t *)&(aci_evt->params.timing.conn_rf_interval), /* Byte aligned */
//                                PIPE_UART_OVER_BTLE_UART_LINK_TIMING_CURRENT_SET_MAX_SIZE);
//        break;
//
//      case ACI_EVT_DISCONNECTED:
//          sent = false;
//        printf("Evt Disconnected/Advertising timed out\r\n");
//        lib_aci_connect(0/* in seconds  : 0 means forever */, 0x0050 /* advertising interval 50ms*/);
//        printf("Advertising started. Tap Connect on the nRF UART app\r\n");
//        break;
//
//      case ACI_EVT_DATA_RECEIVED:
//        printf("Pipe Number: ");
//        printf("%d\r\n",aci_evt->params.data_received.rx_data.pipe_number);
//
//        if (PIPE_UART_OVER_BTLE_UART_RX_RX == aci_evt->params.data_received.rx_data.pipe_number)
//          {
//
//            printf(" Data(Hex) : ");
//            int i;
//            for(i=0; i<aci_evt->len - 2; i++)
//            {
//              printf("%X ",(char)aci_evt->params.data_received.rx_data.aci_data[i]);
//              uart_buffer[i] = aci_evt->params.data_received.rx_data.aci_data[i];
//             // Serial.print(F(" "));
//            }
//            uart_buffer_len = aci_evt->len - 2;
//           // Serial.println(F(""));
////            completed = false;
//            if (lib_aci_is_pipe_available(&aci_state, PIPE_UART_OVER_BTLE_UART_TX_TX))
//            {
//              /*Do this to test the loopback otherwise comment it out*/
//              /*
//              if (!uart_tx(&uart_buffer[0], aci_evt->len - 2))
//              {
//                Serial.println(F("UART loopback failed"));
//              }
//              else
//              {
//                Serial.println(F("UART loopback OK"));
//              }
//              */
//            }
//        }
//        if (PIPE_UART_OVER_BTLE_UART_CONTROL_POINT_RX == aci_evt->params.data_received.rx_data.pipe_number)
//        {
//          uart_process_control_point_rx(&aci_evt->params.data_received.rx_data.aci_data[0], aci_evt->len - 2); //Subtract for Opcode and Pipe number
//        }
//        break;
//
//      case ACI_EVT_DATA_CREDIT:
//        aci_state.data_credit_available = aci_state.data_credit_available + aci_evt->params.data_credit.credit;
//        break;
//
//      case ACI_EVT_PIPE_ERROR:
//        //See the appendix in the nRF8001 Product Specication for details on the error codes
//        printf("ACI Evt Pipe Error: Pipe #:");
//        printf("%d\r\n",aci_evt->params.pipe_error.pipe_number);
//        printf("  Pipe Error Code: 0x");
//        printf("%x \r\n",aci_evt->params.pipe_error.error_code);
//
//        //Increment the credit available as the data packet was not sent.
//        //The pipe error also represents the Attribute protocol Error Response sent from the peer and that should not be counted
//        //for the credit.
//        if (ACI_STATUS_ERROR_PEER_ATT_ERROR != aci_evt->params.pipe_error.error_code)
//        {
//          aci_state.data_credit_available++;
//        }
//        break;
//
//      case ACI_EVT_HW_ERROR:
//        printf("HW error: ");
//        printf("%d \r\n",aci_evt->params.hw_error.line_num);
//
//        uint8_t counter;
//        for(counter = 0; counter <= (aci_evt->len - 3); counter++)
//        {
//          printf("%d ", aci_evt->params.hw_error.file_name[counter]); //uint8_t file_name[20];  //was a arduino write???
//        }
//        printf("\r\n");
//        lib_aci_connect(0/* in seconds, 0 means forever */, 0x0050 /* advertising interval 50ms*/);
//        printf("Advertising started. Tap Connect on the nRF UART app");
//        break;
//
//    }
//  }
//  else
//  {
//      uint8_t data[] = "250";
//      if ((lib_aci_is_pipe_available(&aci_state, PIPE_UART_OVER_BTLE_UART_TX_TX)) && completed == false)
//      {
//          printf("progress");
//          uart_tx(data, strlen(data));
//          sent = true;
//      }
//  }
//  /* setup_required is set to true when the device starts up and enters setup mode.
//   * It indicates that do_aci_setup() should be called. The flag should be cleared if
//   * do_aci_setup() returns ACI_STATUS_TRANSACTION_COMPLETE.
//   */
//  if(setup_required)
//  {
//    if (SETUP_SUCCESS == do_aci_setup(&aci_state))
//    {
//      setup_required = false;
//      vio
//    }
//  }
//  return sent;
//}

bool stringComplete = false;  // whether the string is complete
uint8_t stringIndex = 0;      //Initialize the index to store incoming chars

void serialEvent(void);  // prototype for fnc below

//bool loop(bool completed) {
//
//  //Process any ACI commands or events
//  bool sent = aci_loop(completed);
//
//  // print the string when a newline arrives:
//  if (stringComplete)
//  {
//    printf("Sending: ");
//    printf("%c \r\n",(char *)&uart_buffer[0]);
//
//    uart_buffer_len = stringIndex + 1;
//
//    if (!lib_aci_send_data(PIPE_UART_OVER_BTLE_UART_TX_TX, uart_buffer, uart_buffer_len))
//    {
//      printf("Serial input dropped\r\n");
//    }
//
//    // clear the uart_buffer:
//    for (stringIndex = 0; stringIndex < 20; stringIndex++)
//    {
//      uart_buffer[stringIndex] = ' ';
//    }
//
//    // reset the flag and the index in order to receive more data
//    stringIndex    = 0;
//    stringComplete = false;
//  }
  //For ChipKit you have to call the function that reads from Serial
//  #if defined (__PIC32MX__)
//    if (U6STAbits.URXDA)
//    {
//      serialEvent();
//    }
//  #endif
//
//  return sent;
//}

/*
 COMMENT ONLY FOR ARDUINO
 SerialEvent occurs whenever a new data comes in the
 hardware serial RX.  This routine is run between each
 time loop() runs, so using delay inside loop can delay
 response.  Multiple bytes of data may be available.
 Serial Event is NOT compatible with Leonardo, Micro, Esplora
 */
void serialEvent(void) {

  while(U6STAbits.URXDA){
    // get the new byte:
    dummychar = (uint8_t)getu();
    if(!stringComplete)
    {
      if (dummychar == '\n')
      {
        // if the incoming character is a newline, set a flag
        // so the main loop can do something about it
        stringIndex--;
        stringComplete = true;
      }
      else
      {
        if(stringIndex > 19)
        {
          printf("Serial input truncated\r\n");
          stringIndex--;
          stringComplete = true;
        }
        else
        {
          // add it to the uart_buffer
          uart_buffer[stringIndex] = dummychar;
          stringIndex++;
        }
      }
    }
  }
}

//////////////////////////////////////////////////////////////////////////////////
struct aci_return {
    bool sent;
    int count;
    bool connected;

};


struct aci_return aci_loop(struct aci_return results, float data)
{

    static bool setup_required = false;
//  bool completed = false;

  // We enter the if statement only when there is a ACI event available to be processed
  if (lib_aci_event_get(&aci_state, &aci_data))
  {
    aci_evt_t * aci_evt;
    aci_evt = &aci_data.evt;
    switch(aci_evt->evt_opcode)
    {
      /**
      As soon as you reset the nRF8001 you will get an ACI Device Started Event
      */
      case ACI_EVT_DEVICE_STARTED:
      {
        aci_state.data_credit_total = aci_evt->params.device_started.credit_available;
        switch(aci_evt->params.device_started.device_mode)
        {
          case ACI_DEVICE_SETUP:
            /**
            When the device is in the setup mode
            */
            printf("Evt Device Started: Setup\r\n");
            setup_required = true;
            break;

          case ACI_DEVICE_STANDBY:
            printf("Evt Device Started: Standby\r\n");
            //Looking for an iPhone by sending radio advertisements
            //When an iPhone connects to us we will get an ACI_EVT_CONNECTED event from the nRF8001
            if (aci_evt->params.device_started.hw_error)
            {
              delay_ms(20); //Handle the HW error event correctly.
            }
            else
            {
              lib_aci_connect(0/* in seconds : 0 means forever */, 0x0050 /* advertising interval 50ms*/);
              printf("Advertising started : Tap Connect on the nRF UART app\r\n");
            }

            break;
        }
      }
      break; //ACI Device Started Event

      case ACI_EVT_CMD_RSP:
        //If an ACI command response event comes with an error -> stop
        if (ACI_STATUS_SUCCESS != aci_evt->params.cmd_rsp.cmd_status)
        {
          //ACI ReadDynamicData and ACI WriteDynamicData will have status codes of
          //TRANSACTION_CONTINUE and TRANSACTION_COMPLETE
          //all other ACI commands will have status code of ACI_STATUS_SCUCCESS for a successful command
//          Serial.print(F("ACI Command "));
//          Serial.println(aci_evt->params.cmd_rsp.cmd_opcode, HEX);
//          Serial.print(F("Evt Cmd respone: Status "));
//          Serial.println(aci_evt->params.cmd_rsp.cmd_status, HEX);

          printf("ACI Command ");
          printf("%x\r\n",aci_evt->params.cmd_rsp.cmd_opcode);
          printf("Evt Cmd respone: Status ");
          printf("%x\r\n",aci_evt->params.cmd_rsp.cmd_status);
        }
        if (ACI_CMD_GET_DEVICE_VERSION == aci_evt->params.cmd_rsp.cmd_opcode)
        {
          //Store the version and configuration information of the nRF8001 in the Hardware Revision String Characteristic
          lib_aci_set_local_data(&aci_state, PIPE_DEVICE_INFORMATION_HARDWARE_REVISION_STRING_SET,
            (uint8_t *)&(aci_evt->params.cmd_rsp.params.get_device_version), sizeof(aci_evt_cmd_rsp_params_get_device_version_t));
        }
        break;

      case ACI_EVT_CONNECTED:
        results.sent = false;
        results.connected = true;
        printf("Evt Connected\n\r");
        uart_over_ble_init();
        timing_change_done              = false;
        aci_state.data_credit_available = aci_state.data_credit_total;
//        completed = false;
        /*
        Get the device version of the nRF8001 and store it in the Hardware Revision String
        */
        lib_aci_device_version();
        break;

      case ACI_EVT_PIPE_STATUS:
        printf("Evt Pipe Status\n\r");
        results.count += results.count;
        //completed = false;
        if (lib_aci_is_pipe_available(&aci_state, PIPE_UART_OVER_BTLE_UART_TX_TX) && (false == timing_change_done))
        {
          lib_aci_change_timing_GAP_PPCP(); // change the timing on the link as specified in the nRFgo studio -> nRF8001 conf. -> GAP.
                                            // Used to increase or decrease bandwidth
          timing_change_done = true;

          int data_int;
          data_int = (int)data;
          int str_size;
          str_size = sizeof(int) * 3 + 2;
          char data_string[str_size];
          
          snprintf(data_string,sizeof(data_string),"%d",data_int);

          //char hello[]="Hello World, works";
          //uart_tx((uint8_t *)&hello[0], strlen(hello));
          uart_tx((uint8_t *)&data_string[0], strlen(data_string));
          printf("Sending: ");
          printf("%d\r\n",data_int);
        }
        char data[] = "connected";
        uart_tx((uint8_t *)&data, strlen(data));
        break;

      case ACI_EVT_TIMING:
        printf("Evt link connection interval changed");
        lib_aci_set_local_data(&aci_state,
                                PIPE_UART_OVER_BTLE_UART_LINK_TIMING_CURRENT_SET,
                                (uint8_t *)&(aci_evt->params.timing.conn_rf_interval), /* Byte aligned */
                                PIPE_UART_OVER_BTLE_UART_LINK_TIMING_CURRENT_SET_MAX_SIZE);
        break;

      case ACI_EVT_DISCONNECTED:
        results.sent = false;
        printf("Evt Disconnected/Advertising timed out\r\n");
        lib_aci_connect(0/* in seconds  : 0 means forever */, 0x0050 /* advertising interval 50ms*/);
        printf("Advertising started. Tap Connect on the nRF UART app\r\n");
        break;

      case ACI_EVT_DATA_RECEIVED:
        printf("Pipe Number: ");
        printf("%d\r\n",aci_evt->params.data_received.rx_data.pipe_number);

        if (PIPE_UART_OVER_BTLE_UART_RX_RX == aci_evt->params.data_received.rx_data.pipe_number)
          {

            printf(" Data(Hex) : ");
            int i;
            for(i=0; i<aci_evt->len - 2; i++)
            {
              printf("%X ",(char)aci_evt->params.data_received.rx_data.aci_data[i]);
              uart_buffer[i] = aci_evt->params.data_received.rx_data.aci_data[i];
             // Serial.print(F(" "));
            }
            uart_buffer_len = aci_evt->len - 2;
           // Serial.println(F(""));
//            completed = false;
            if (lib_aci_is_pipe_available(&aci_state, PIPE_UART_OVER_BTLE_UART_TX_TX))
            {
              /*Do this to test the loopback otherwise comment it out*/
              /*
              if (!uart_tx(&uart_buffer[0], aci_evt->len - 2))
              {
                Serial.println(F("UART loopback failed"));
              }
              else
              {
                Serial.println(F("UART loopback OK"));
              }
              */
            }
        }
        if (PIPE_UART_OVER_BTLE_UART_CONTROL_POINT_RX == aci_evt->params.data_received.rx_data.pipe_number)
        {
          uart_process_control_point_rx(&aci_evt->params.data_received.rx_data.aci_data[0], aci_evt->len - 2); //Subtract for Opcode and Pipe number
        }
        break;

      case ACI_EVT_DATA_CREDIT:
        aci_state.data_credit_available = aci_state.data_credit_available + aci_evt->params.data_credit.credit;
        break;

      case ACI_EVT_PIPE_ERROR:
        //See the appendix in the nRF8001 Product Specication for details on the error codes
        printf("ACI Evt Pipe Error: Pipe #:");
        printf("%d\r\n",aci_evt->params.pipe_error.pipe_number);
        printf("  Pipe Error Code: 0x");
        printf("%x \r\n",aci_evt->params.pipe_error.error_code);

        //Increment the credit available as the data packet was not sent.
        //The pipe error also represents the Attribute protocol Error Response sent from the peer and that should not be counted
        //for the credit.
        if (ACI_STATUS_ERROR_PEER_ATT_ERROR != aci_evt->params.pipe_error.error_code)
        {
          aci_state.data_credit_available++;
        }
        break;

      case ACI_EVT_HW_ERROR:
        printf("HW error: ");
        printf("%d \r\n",aci_evt->params.hw_error.line_num);

        uint8_t counter;
        for(counter = 0; counter <= (aci_evt->len - 3); counter++)
        {
          printf("%d ", aci_evt->params.hw_error.file_name[counter]); //uint8_t file_name[20];  //was a arduino write???
        }
        printf("\r\n");
        lib_aci_connect(0/* in seconds, 0 means forever */, 0x0050 /* advertising interval 50ms*/);
        printf("Advertising started. Tap Connect on the nRF UART app");
        break;

    }
  }
  else
  {
      char data_tx[10];
      snprintf(data_tx, sizeof(data_tx), "%f", data);
//      char data = I2C_read();
      if ((lib_aci_is_pipe_available(&aci_state, PIPE_UART_OVER_BTLE_UART_TX_TX)) && results.count < 10)
      {
          uart_tx((uint8_t*)&data_tx, strlen(data_tx));
          printf("transfer");
          results.count = results.count + 1;
          printf("%d",results.count);
      }
  }
  /* setup_required is set to true when the device starts up and enters setup mode.
   * It indicates that do_aci_setup() should be called. The flag should be cleared if
   * do_aci_setup() returns ACI_STATUS_TRANSACTION_COMPLETE.
   */
  if(setup_required)
  {
    if (SETUP_SUCCESS == do_aci_setup(&aci_state))
    {
      setup_required = false;
      vio
    }
  }

  if(results.count > 1)
      results.sent = true;
//      lib_aci_disconnect(&aci_state, ACI_REASON_TERMINATE);

  return results;
}

////////////////////////////////////////////////////////////////////////////////

//bool loop(bool completed) {
//
//  //Process any ACI commands or events
//  bool sent = aci_loop(struct aci_return results);
//
////  // print the string when a newline arrives:
////  if (stringComplete)
////  {
////    printf("Sending: ");
////    printf("%c \r\n",(char *)&uart_buffer[0]);
////
////    uart_buffer_len = stringIndex + 1;
////
////    if (!lib_aci_send_data(PIPE_UART_OVER_BTLE_UART_TX_TX, uart_buffer, uart_buffer_len))
////    {
////      printf("Serial input dropped\r\n");
////    }
////
////    // clear the uart_buffer:
////    for (stringIndex = 0; stringIndex < 20; stringIndex++)
////    {
////      uart_buffer[stringIndex] = ' ';
////    }
////
////    // reset the flag and the index in order to receive more data
////    stringIndex    = 0;
////    stringComplete = false;
////  }
////  #if defined (__PIC32MX__)
////    if (U6STAbits.URXDA)
////    {
////      serialEvent();
////    }
////  #endif
//
//  return sent;
//}

bool ble_connect(float data) {
    struct aci_return results;
    results.sent = false;
    results.count = 0;
    results.connected = false;

    while(results.sent == false)
    {
       //printf("false...\r\n");
       results = aci_loop(results, data);
       if(results.connected == true) {
           printf("connected \n");
           break;
       }
    }

    return results.connected;
}

bool ble_transfer(float data) {
    struct aci_return results;
    results.sent = false;
    results.count = 0;
    //results.connected = false;

    while(results.sent == false)
    {
       //printf("false...\r\n");
       results = aci_loop(results, data);
       //printf("results.sent: %d\n\r",results.sent);
       if(results.sent == true)
           printf("sent test 1 \n");
    }

    return results.sent;
}





#endif	/* BLE_H */

